      subroutine findtpsi(n,irc,energy,l,rveff,h,v0,v0p,proj,nmx,       &
        hij,qij,mxb,many,psi,tpsi)
!  pgm to solve separable radial schroedinger equation for bound state
!    at the energy 'energy' and at angular momentum l
!
! assume energy appropriate for bound function is given by energy (no iteration)
!
!    with potential rveff/r, given in uniform mesh of n points
!   r=i*h, i=1,...n-1 ;assuming p(r)=C*r**(l+1)*polynomial(r) for r==0;
!                               p((n+1)*h)=0
!
!  uses Noumerov algorithm
!
!  For l=0,1 corrections are needed to approximate wfn(r=0)
!     These depend upon:
!         e0 (current guess of energy eigenvalue)
!         l
!         v(0) == v0 electronic potential at r=0
!         v'(0) == v0p derivative of electronic potential at r=0
!
!  proj == projector functions
!  hij and qij == hamiltonianian and overlap matrix elements
!
      implicit real*8 (a-h,o-z)
      parameter (convre=1.d-10,vlrg=1.d30,inc=10)
      dimension rveff(n),psi(n),tpsi(n),proj(nmx,many),y(nmx,many),u(nmx)
      dimension a(mxb,mxb),b(mxb),hij(mxb,mxb),qij(mxb,mxb)
      dimension d1(nmx),d2(nmx)

      if (n.gt.nmx) then
       write(6,*) 'error in findtpsi -- n,nmx = ',n,nmx
       stop
       endif
      !write(6,*) 'in findtpsi', l,many,irc,n,energy,h
!
!
!  perform outward integration until match point -- it is assumed
!   that projector functions proj are zero for r>r(match)
!
      mmatch=irc+2
      noz=0
      call outwardh(mmatch,energy,u,l,noz,rveff,h,v0,v0p,node,          &
          nmx,d1,d2)
!
      do ib=1,many
      call outwardi(mmatch,energy,y(1,ib),l,rveff,proj(1,ib),h,         &
        nmx,d1,d2)
      enddo
!
      do ib=1,many
      sum=0.d0
       do ic=1,many
       sum=sum+(hij(ib,ic)-energy*qij(ib,ic))*                          &
                 overlap(mmatch,h,proj(1,ic),u)
       enddo
      b(ib)=-sum
!     write(6,*) 'ib,sum',ib,sum
      enddo
!
      do ia=1,many
      do ib=1,many
      sum=0.d0
      if (ia.eq.ib) sum=1.d0
       do ic=1,many
       sum=sum+(hij(ia,ic)-energy*qij(ia,ic))*                          &
                 overlap(mmatch,h,proj(1,ic),y(1,ib))
       enddo
      a(ia,ib)=sum
!     write(6,*) 'ia,ib,sum',ia,ib,sum
      enddo
      enddo
!
      call linsol(a,b,mxb,many,1)
!     write(6,*) 'b(ib)=',(b(ib),ib=1,many)

      tpsi(1:n)=psi(1:n)

      do ib=1,many
      u(1:mmatch)=u(1:mmatch)+b(ib)*y(1:mmatch,ib)
      enddo
!
      rout=(u(irc+1)-u(irc-1))/(2*h*u(irc))
      rin= (psi(irc+1)-psi(irc-1))/(2*h*psi(irc))

      write(6,*) 'in findtpsi  -- rout rin = ',rout,rin

       xx=psi(irc)/u(irc)

       tpsi(1:irc)=xx*u(1:irc)

!  check normalization

           !write(6,*) 'calling sepnorm',n,h,many
          scale=sepnorm(n,h,tpsi,nmx,many,proj,b,mxb,qij)

           write(6,*) 'checking normalization',scale
!
      return
       end
